##
## clirc -- An IRC client in Python.
## curseseditline.py - Curses almost single line editor.
##
## Copyleft 1997-2003 Teemu Kalvas <chery@s2.org>
##

##import linelist
import reloader

reloader.register_module('curseseditline')

class CursesEditline(reloader.ValueDefaulter):
    def __init__(self, screen):
        self.screen = screen
        self.prompt = '> '
        ##self.history = linelist.LineList(8)
        self.history = 1024 * [None]
        self.history_pos = 0
        self.history_end = 0
        self.height = 1
        self.reset()

    def reset(self, *rest):
        self.string = ''
        self.offset = -len(self.prompt)
        self.position = 0
        self.redraw()

    def maybe_set_height(self):
        width = self.screen.width
        h = 1 + (len(self.prompt) + len(self.string)) / width
        if h <> self.height:
            self.set_height(h)

    def set_height(self, height):
        self.height = height
        self.screen.set_partition_height(self, height)

    def insert(self, str):
        width = self.screen.width
        a = (len(self.prompt) + len(self.string)) / width;
        self.string = self.string[:self.position] + str \
                      + self.string[self.position:]
        b = (len(self.prompt) + len(self.string)) / width;
        c = (len(self.prompt) + self.position) / width;
	self.position = self.position + len(str)
        if a == b and a == c:
            window = self.window()
	    x = self.position + len(self.prompt) - len(str)
            y = x / width;
            x = x % width;
	    for ch in str:
		window.insch(y, x, ord(ch))
		x = x + 1
            self.changed()
        else:
            self.redraw()

    def set_string(self, str):
        self.string = str
        self.position = len(str)
        self.offset = -len(self.prompt)
        self.reposition()
        self.redraw()

    def accept(self, event):
        ##self.history.append(self.string)
        ##self.history_pos = self.history.lines
        self.history[self.history_end] = self.string
        self.history_end = self.history_end + 1
        if self.history_end >= len(self.history):
            self.history_end = 0
        self.history_pos = self.history_end
        event.accept(self.string)
        self.reset()

    def refresh(self, event):
        self.screen.refresh()

    def del_char_left(self, event):
        if self.position:
            self.position = self.position - 1
            self.string = self.string[:self.position] \
                          + self.string[self.position + 1:]
            self.redraw()

    def del_char_right(self, event):
        if self.position < len(self.string):
            self.string = self.string[:self.position] \
                          + self.string[self.position + 1:]
            self.redraw()

    def line_beg(self, event):
        self.position = 0
        self.offset = -len(self.prompt)
        self.reposition()

    def line_end(self, event):
        self.position = len(self.string)
        self.reposition()

    def char_left(self, event):
        if self.position:
            self.position = self.position - 1
            self.reposition()

    def char_right(self, event):
        if self.position < len(self.string):
            self.position = self.position + 1
            self.reposition()

    def del_line_end(self, event):
        self.string = self.string[:self.position]
        self.redraw()

    def history_prev(self, event):
        width = self.screen.width
        if self.position >= width:
            self.position = self.position - width
            self.reposition()
        else:
            self.history_prev_entry(event)

    def history_prev_entry(self, event):
        i = self.history_pos - 1
        if i < 0:
            i = len(self.history) - 1
        if self.history[i] <> None:
            self.history_pos = i
            self.set_string(self.history[i])

    def history_next(self, event):
        width = self.screen.width
        if self.position <= len(self.string) - width:
            self.position = self.position + width
            self.reposition()
        else:
            self.history_next_entry(event)

    def history_next_entry(self, event):
        if self.history_pos <> self.history_end:
            self.history_pos = self.history_pos + 1
            if self.history_pos >= len(self.history):
                self.history_pos = 0
            if self.history_pos == self.history_end:
                self.set_string('')
            else:
                self.set_string(self.history[self.history_pos])

    def transpose_char(self, event):
        str = self.string
        pos = self.position
        if len(str) >= 2:
            if pos == len(str):
                self.string = str[:pos - 2] + str[pos - 1] + str[pos - 2]
            elif pos == 0:
                self.string = str[1] + str[0] + str[2:]
            else:
                self.string = str[:pos - 1] + str[pos] + str[pos - 1] \
                              + str[pos + 1:]
                self.position = pos + 1
            self.redraw()

    def window(self):
        return self.screen.get_window(self)

    def changed(self):
        self.screen.window_changed(self)
        self.cursor_changed()

    def cursor_changed(self):
        ##self.screen.cursor_changed(self, self.position - self.offset)
        width = self.screen.width
        pos = self.position + len(self.prompt)
        self.screen.cursor_changed(self, pos % width, pos / width)

    def reposition(self):
	self.cursor_changed()
	width = self.screen.width
        if self.position <= self.offset + 2 or \
           self.position >= self.offset + width - 2:
            self.redraw()

    def redraw(self):
        self.maybe_set_height()
        self.screen.redraw_window(self)

    def redraw_event(self):
        window = self.window()
        width = self.screen.width
        ##if self.position <= self.offset + 2 or \
        ##   self.position >= self.offset + width - 2:
        ##    self.offset = max(-len(self.prompt), self.position - width / 2)
        str = self.prompt + self.string
        ##off = self.offset + len(self.prompt)
        ##if len(str) - off >= width:
        ##    window.addstr(0, 0, str[off:off + width - 1])
        ##else:
        ##    window.addstr(0, 0, str[off:])
        ##    window.clrtoeol()
        a = 0
        y = 0
        while a <= len(str):
            b = a + width
            if b < len(str):
                window.addstr(y, 0, str[a:b])
            else:
                window.addstr(y, 0, str[a:])
                #window.addstr(y, 0, str[a:] + (b - len(str) - 1) * ' ')
                window.clrtoeol()
            a = b
            y = y + 1
        self.changed()

## End. ##
